class ChoiceMaster {
  // Array that holds the IDs of the products selected
  int[] m_selectedProducts;

  // Number of products currently selected
  int m_numProducts;

  // Page displayed
  int m_pageDisplayed;

  // UI measurements
  float m_fullHeight;
  float m_placeY;
  float m_chooseProductsButtonsHeight;
  float m_chooseProductsWidth;
  float m_chooseProductsPlaceX;  

  // Submenus
  ChoiceSub m_chooseIngredients;
  ChoiceSub m_chooseAlternatives;

  // Buttons for navigation and selection
  AddProductButton b_add;
  ViewChartButton b_chart;
  PageButton b_up;
  PageButton b_down;
  ChooseProductButton[] b_chooseProductButtons;
  RemoveProductButton[] b_removeProduct;
  Servings servingsMenu;

  // Constructor
  public ChoiceMaster( int _sizeGroup ) {
    m_selectedProducts = new int[5];
    m_numProducts = 0;
    m_pageDisplayed = 0;

    // Measurements for UI elements
    m_placeY = 71;
    m_fullHeight = 654;
    m_chooseProductsWidth = 382;
    m_chooseProductsButtonsHeight = 62;
    m_chooseProductsPlaceX = 62;

    // Submenus
    m_chooseIngredients = new ChoiceSub( 475 );
    m_chooseAlternatives = new ChoiceSub( 750 );

    // Buttons to navigate the UI
    b_add = new AddProductButton();
    b_chart = new ViewChartButton();
    b_up = new PageButton( true );
    b_down = new PageButton( false );

    //Buttons to represent the product list
    b_chooseProductButtons = new ChooseProductButton[_sizeGroup];
    b_removeProduct = new RemoveProductButton[5];
    for (int i = 0; i < 5; i++ ) {
      b_removeProduct[i] = new RemoveProductButton();
    }


    servingsMenu = new Servings();
  }

  void createProductButtons( ArrayList _products ) {
    for ( int i = 0; i < _products.size(); i++ ) {
      Product thisProduct = (Product) _products.get( i );
      b_chooseProductButtons[i] = new ChooseProductButton(
      thisProduct.m_id, 
      split( thisProduct.m_description, "\\(" )[0], 
      thisProduct.m_servingSize, 
      -500, -400, 
      m_chooseProductsWidth, 
      m_chooseProductsButtonsHeight );
    }
  }

  // Leftmost menu of product selections
  void renderSelections() {
    renderSelectedProductButtons();
    renderFunctionButtons();
    renderRemoveProductButtons();
  }

  // Show the list of selected product buttons
  void renderSelectedProductButtons() {
    if ( m_numProducts > 0 ) {
      for ( int i = 0; i < b_chooseProductButtons.length; i++ ) {
        if ( b_chooseProductButtons[i].m_selected == true ) {
          b_chooseProductButtons[i].renderButton( m_chooseProductsPlaceX + m_chooseProductsWidth / 2, m_placeY + m_chooseProductsButtonsHeight / 2 + m_chooseProductsButtonsHeight * b_chooseProductButtons[i].m_positionIndex );
        }
      }
    }
  }

  // Show the add and chart buttons
  void renderFunctionButtons() {
    pushMatrix();
    translate( m_chooseProductsPlaceX + m_chooseProductsWidth / 2 - 33 / 2, m_placeY + m_chooseProductsButtonsHeight * m_numProducts + m_chooseProductsButtonsHeight / 2 );
    // Shift right if there is nothing to put in the chart view
    if ( m_numProducts <= 0 ) {
      translate ( 10, 0 );
    }
    b_add.render( 0, 0 );
    if ( m_numProducts > 0 ) {
      pushMatrix();
      translate( 20, 0 );
      b_chart.render( 0, 0 );
      popMatrix();
    }
    popMatrix();
    line( m_chooseProductsPlaceX, m_placeY + m_chooseProductsButtonsHeight * ( m_numProducts + 1 ), m_chooseProductsPlaceX + m_chooseProductsWidth, 
    m_placeY + m_chooseProductsButtonsHeight * (m_numProducts + 1 ) );
  }

  void renderRemoveProductButtons() {
    for ( int i = 0; i < m_numProducts; i++ ) {
      pushMatrix();
      translate( m_chooseProductsPlaceX + m_chooseProductsWidth - 2, m_placeY + m_chooseProductsButtonsHeight * i + m_chooseProductsButtonsHeight / 2 );
      b_removeProduct[i].render( 0, 0 );
      popMatrix();
    }
  }


  // Show the list of poosible product selections
  void renderProductOptions() {
    renderProductButtons();
    renderPageButtons();
  }

  // Show a button for each product
  void renderProductButtons() {
    // Count the number of buttons displayed and position accordingly
    int count = 0;
    // If we're not on the last page
    if ( m_pageDisplayed < floor ( b_chooseProductButtons.length / 10 ) ) {
      for ( int i = 10 * m_pageDisplayed; i < 10 + 10 * m_pageDisplayed; i++ ) {
        b_chooseProductButtons[i].renderButton( m_chooseProductsPlaceX + m_chooseProductsWidth / 2, m_placeY + m_chooseProductsButtonsHeight / 2 + m_chooseProductsButtonsHeight * count );
        count++;
      }
    }
    //If we are on the last page
    else {
      for (int i = 10 * m_pageDisplayed; i < b_chooseProductButtons.length; i++ ) {
        b_chooseProductButtons[i].renderButton( m_chooseProductsPlaceX + m_chooseProductsWidth / 2, m_placeY + m_chooseProductsButtonsHeight / 2 + m_chooseProductsButtonsHeight * count );
        count++;
      }
    }
  }

  void renderPageButtons() {
    pushMatrix();
    translate( m_chooseProductsPlaceX + m_chooseProductsWidth / 2 - 33 / 2, m_placeY + m_chooseProductsButtonsHeight / 3 + m_chooseProductsButtonsHeight * 10 );
    b_up.render ( 0, 0 );
    pushMatrix();
    translate( 20, 0 );
    b_down.render( 0, 0 );
    popMatrix();
    popMatrix();
  }

  void resetProductButtons() {
    int count = 0;
    // If we're not ont he last page
    if ( m_pageDisplayed < floor( b_chooseProductButtons.length / 10 ) ) {
      for ( int i = 10 * m_pageDisplayed; i < 10 + 10 * m_pageDisplayed; i++ ) {
        b_chooseProductButtons[i].resetPosition( -500, -500 );
        count++;
      }
    }
    else {
      for ( int i = 10 * m_pageDisplayed; i < b_chooseProductButtons.length; i++ ) {
        b_chooseProductButtons[i].resetPosition( -500, -500 );
        count++;
      }
    }
  }

  //Background elements of the Choice screen
  void renderBackground() {
    // Text color
    fill( 0 );
    noStroke();
    //Choice screen header
    textAlign( LEFT );
    textFont( abel32 );
    text( "What Should I Eat?", 80, 55 );
    stroke( 0 );
    noFill();
    line( m_chooseProductsPlaceX, m_placeY, m_chooseProductsWidth + m_chooseProductsPlaceX, m_placeY );

    //Serving Size Bar header
    textAlign( CENTER );
    textFont( abel12 );
    fill( 0 );
    noStroke();
    text( "Serving", 36, 48 );
    text( "Size", 36, 63 );

    //Serving Size Bar background
    fill( 77, 77, 77 );
    noStroke();
    rect( 24, m_placeY, 25, m_fullHeight );

    //Middle and Right Menu outlines
    m_chooseIngredients.renderBackground();
    m_chooseAlternatives.renderBackground();
  } 

  //white rectangles to hide the menu scrolling
  void hideMenuScrolling() {
    m_chooseIngredients.hideMenuScrolling();
    m_chooseAlternatives.hideMenuScrolling();
  }


  // check for mouse interaction with the page buttons
  void checkPageButtons() {
    // If we clicked on the page down button
    if (
    mouseX >= b_down.m_screenX - 7 &&
      mouseX <= b_down.m_screenX + 7 &&
      mouseY >= b_down.m_screenY - 7 &&
      mouseY <= b_down.m_screenY + 7
      && m_pageDisplayed <= 12 ) {

      // Increment the page displayed
      m_pageDisplayed++;
      // Make sure we don't scroll off the list
      if ( m_pageDisplayed > floor( b_chooseProductButtons.length / 10 ) ) {
        m_pageDisplayed = floor( b_chooseProductButtons.length / 10 );
      }
    }

    // If we clicked on the page up button
    if (
    mouseX >= b_up.m_screenX - 7 &&
      mouseX <= b_up.m_screenX + 7 &&
      mouseY >= b_up.m_screenY - 7 &&
      mouseY <= b_up.m_screenY + 7
      && m_pageDisplayed >= 1 ) {

      // Devrement the page displayed
      m_pageDisplayed--;
      // Make sure we don't scroll off the list
      if ( m_pageDisplayed < 0 ) m_pageDisplayed = 0;
    }
  }


  void selectIngredientList( int _i ) {
    for (int i = 0; i < products.size(); i++) {
      Product thisProduct = (Product) products.get(i);
      if ( thisProduct.m_id == b_chooseProductButtons[_i].m_id ) {
        m_chooseIngredients.m_productNum = i;
        m_chooseIngredients.m_heading = thisProduct.m_description;
        m_chooseIngredients.createIngredientButtons();
        continue;
      }
    }
    m_chooseIngredients.m_display = true;
  }


  // check for mouse interaction with the product buttons
  void checkProductButtons() {
    // If we're not on the last page of product buttons
    if ( m_pageDisplayed < floor( b_chooseProductButtons.length / 10 ) ) {
      // Check each button
      for ( int i = 10 * m_pageDisplayed; i < 10 + 10 * m_pageDisplayed; i++ ) {
        checkProductButton( i );
      }
    }
    // If we're on the last page of product buttons
    else if ( m_pageDisplayed >= floor( b_chooseProductButtons.length / 10 ) ) {
      // Check each button
      for ( int i = 10 * m_pageDisplayed; i < b_chooseProductButtons.length; i++ ) {
        checkProductButton( i );
      }
    }
  }

  void checkProductButton(int _i) {
    // If click is detected inside one of the displayed buttons
    if (
    mouseX >= b_chooseProductButtons[_i].m_buttonLeft &&
      mouseX <= b_chooseProductButtons[_i].m_buttonRight - 10 &&    //remove 10 pixels from the right for the remove product button
    mouseY >= b_chooseProductButtons[_i].m_buttonTop &&
      mouseY <= b_chooseProductButtons[_i].m_buttonBottom ) {
      //println( b_chooseProductButtons[i].m_productName );

      m_chooseAlternatives.m_display = false;

      if ( addProduct == true ) {
        // Check to see if this product has already been selected
        boolean alreadySelected = false;
        for ( int j = 0; j < m_selectedProducts.length; j++) {
          if ( m_selectedProducts[j] == b_chooseProductButtons[_i].m_id ) {
            alreadySelected = true;
          }
        }

        // If the product hasn't been selected
        if ( alreadySelected == false ) {

          // Error check for more than 5 products
          if ( m_numProducts >= 4 ) {
            // Loop through all the buttons
            for ( int j = 0; j < b_chooseProductButtons.length; j++) {
              // Find the fifth selections and unselect it
              if ( m_selectedProducts[4] == b_chooseProductButtons[j].m_id ) {
                b_chooseProductButtons[j].m_selected = false;
                b_chooseProductButtons[j].m_positionIndex = -10;

                // Get the list ready to replace the fifth selected button
                m_numProducts = 4;
              }
            }
          }

          // Move all the buttons offscreen
          resetProductButtons();

          // Update the list of selected products
          m_selectedProducts[m_numProducts] = b_chooseProductButtons[_i].m_id;

          // Tell the button that it has been selected
          b_chooseProductButtons[_i].m_selected = true;

          // Display the ingredients list
          selectIngredientList( _i );

          // Tell the button what number it is in the list
          b_chooseProductButtons[_i].m_positionIndex = m_numProducts;

          // Increment the number of products selected
          m_numProducts++;

          // Change the control structure back to displaying the selected products
          addProduct = false;
          m_chooseAlternatives.m_display = false;
        }
      }
      else if ( addProduct == false ) {
        selectIngredientList( _i );
      }
    }
  }

  void checkRemoveProductButtons() {
    for (int i = 0; i < m_numProducts; i++ ) {
      if ( mouseX >= b_removeProduct[i].m_screenX - 6 && mouseX <= b_removeProduct[i].m_screenX + 6 &&
        mouseY >= b_removeProduct[i].m_screenY - 6 && mouseY <= b_removeProduct[i].m_screenY + 6 ) {

        for (int j = 0; j < b_chooseProductButtons.length; j++) {
          if ( m_selectedProducts[i] == b_chooseProductButtons[j].m_id ) {
            println( b_chooseProductButtons[j].m_id );
            b_chooseProductButtons[j].m_selected = false;
            b_chooseProductButtons[j].m_positionIndex = -10;
          }
        }

        for ( int j = i; j < 4; j++ ) {
          m_selectedProducts[j] = m_selectedProducts[j+1];
        }

        for ( int j = m_numProducts; j < 5; j++ ) {
          m_selectedProducts[j] = -1;
        }

        for ( int j = i; j < m_numProducts; j++ ) {
          for ( int k = 0; k < b_chooseProductButtons.length; k++ ) {
            if ( m_selectedProducts[j] == b_chooseProductButtons[k].m_id ) {
              b_chooseProductButtons[k].m_positionIndex--;
            }
          }
        }

        m_numProducts--;
        if ( m_numProducts < 0 ) m_numProducts = 0;

        resetProductButtons();

        m_chooseIngredients.m_display = false;
        m_chooseAlternatives.m_display = false;
        addProduct = false;
      }
    }
  }

  void checkIngredientButtons() {
    if ( m_chooseIngredients.b_buttons.length > 22 ) {
      m_chooseIngredients.checkPageButtons();
    }

    //If the mouse cursor are within the ingredients box

    if ( m_chooseIngredients.m_pageDisplayed < floor ( m_chooseIngredients.b_buttons.length / 22 ) ) {
      for ( int i = 22 * m_chooseIngredients.m_pageDisplayed; i < 22 + 22 * m_chooseIngredients.m_pageDisplayed; i++ ) {
        if (
        mouseX >= m_chooseIngredients.b_buttons[i].m_buttonLeft &&
          mouseX <= m_chooseIngredients.b_buttons[i].m_buttonRight &&
          mouseY >= m_chooseIngredients.b_buttons[i].m_buttonTop &&
          mouseY <= m_chooseIngredients.b_buttons[i].m_buttonBottom
          ) {

          m_chooseAlternatives.m_description = m_chooseIngredients.b_buttons[i].m_description;
          m_chooseAlternatives.createAlternativeButtons();
        }
      }
    }
    else {
      for ( int i = 22 * m_chooseIngredients.m_pageDisplayed; i < m_chooseIngredients.b_buttons.length; i++ ) {
        if (
        mouseX >= m_chooseIngredients.b_buttons[i].m_buttonLeft &&
          mouseX <= m_chooseIngredients.b_buttons[i].m_buttonRight &&
          mouseY >= m_chooseIngredients.b_buttons[i].m_buttonTop &&
          mouseY <= m_chooseIngredients.b_buttons[i].m_buttonBottom
          ) {

          m_chooseAlternatives.m_description = m_chooseIngredients.b_buttons[i].m_description;
          m_chooseAlternatives.createAlternativeButtons();
        }
      }
    }
  }

  void checkAlternativeButtons() {
    if ( m_chooseAlternatives.b_buttons.length > 22 ) {
      m_chooseAlternatives.checkPageButtons();
    }

    //If the mouse cursor are within the ingredients box
    if ( m_chooseAlternatives.m_pageDisplayed < floor ( m_chooseAlternatives.b_buttons.length / 22 ) ) {    
      for ( int i = 22 * m_chooseAlternatives.m_pageDisplayed; i < 22 + 22 * m_chooseAlternatives.m_pageDisplayed; i++ ) {
        if (
        mouseX >= m_chooseAlternatives.b_buttons[i].m_buttonLeft &&
          mouseX <= m_chooseAlternatives.b_buttons[i].m_buttonRight &&
          mouseY >= m_chooseAlternatives.b_buttons[i].m_buttonTop &&
          mouseY <= m_chooseAlternatives.b_buttons[i].m_buttonBottom
          ) {

          // Check to see if this product has already been selected
          boolean alreadySelected = false;
          for ( int j = 0; j < m_selectedProducts.length; j++) {
            if ( m_selectedProducts[j] == m_chooseAlternatives.b_buttons[i].m_id ) {
              alreadySelected = true;
            }
          }

          // If the product hasn't been selected
          if ( alreadySelected == false ) {

            // Error check for more than 5 products
            if ( m_numProducts >= 4 ) {
              // Loop through all the buttons
              for ( int j = 0; j < b_chooseProductButtons.length; j++) {
                // Find the fifth selections and unselect it
                if ( m_selectedProducts[4] == b_chooseProductButtons[j].m_id ) {
                  b_chooseProductButtons[j].m_selected = false;
                  b_chooseProductButtons[j].m_positionIndex = -10;

                  // Get the list ready to replace the fifth selected button
                  m_numProducts = 4;
                }
              }
            }

            // Move all the buttons offscreen
            resetProductButtons();

            for (int j = 0; j < b_chooseProductButtons.length; j++ ) {
              if ( b_chooseProductButtons[j].m_id == m_chooseAlternatives.b_buttons[i].m_id) {
                // Update the list of selected products
                m_selectedProducts[m_numProducts] = b_chooseProductButtons[j].m_id;                

                // Tell the button that it has been selected
                b_chooseProductButtons[j].m_selected = true;

                // Display the ingredients list
                selectIngredientList( j );

                // Tell the button what number it is in the list
                b_chooseProductButtons[j].m_positionIndex = m_numProducts;

                // Increment the number of products selected
                m_numProducts++;

                // Change the control structure back to displaying the selected products
                addProduct = false;
                m_chooseAlternatives.m_display = false;
              }
            }
          }
        }
      }
    }

    else {
      for ( int i = 22 * m_chooseAlternatives.m_pageDisplayed; i < m_chooseAlternatives.b_buttons.length; i++ ) {
        if (
        mouseX >= m_chooseAlternatives.b_buttons[i].m_buttonLeft &&
          mouseX <= m_chooseAlternatives.b_buttons[i].m_buttonRight &&
          mouseY >= m_chooseAlternatives.b_buttons[i].m_buttonTop &&
          mouseY <= m_chooseAlternatives.b_buttons[i].m_buttonBottom
          ) {

          // Check to see if this product has already been selected
          boolean alreadySelected = false;
          for ( int j = 0; j < m_selectedProducts.length; j++) {
            if ( m_selectedProducts[j] == m_chooseAlternatives.b_buttons[i].m_id ) {
              alreadySelected = true;
            }
          }

          // If the product hasn't been selected
          if ( alreadySelected == false ) {

            // Error check for more than 5 products
            if ( m_numProducts >= 4 ) {
              // Loop through all the buttons
              for ( int j = 0; j < b_chooseProductButtons.length; j++) {
                // Find the fifth selections and unselect it
                if ( m_selectedProducts[4] == b_chooseProductButtons[j].m_id ) {
                  b_chooseProductButtons[j].m_selected = false;
                  b_chooseProductButtons[j].m_positionIndex = -10;

                  // Get the list ready to replace the fifth selected button
                  m_numProducts = 4;
                }
              }
            }

            // Move all the buttons offscreen
            resetProductButtons();

            for (int j = 0; j < b_chooseProductButtons.length; j++ ) {
              if ( b_chooseProductButtons[j].m_id == m_chooseAlternatives.b_buttons[i].m_id) {
                // Update the list of selected products
                m_selectedProducts[m_numProducts] = b_chooseProductButtons[j].m_id;                

                // Tell the button that it has been selected
                b_chooseProductButtons[j].m_selected = true;

                // Display the ingredients list
                selectIngredientList( j );

                // Tell the button what number it is in the list
                b_chooseProductButtons[j].m_positionIndex = m_numProducts;

                // Increment the number of products selected
                m_numProducts++;

                // Change the control structure back to displaying the selected products
                addProduct = false;
                m_chooseAlternatives.m_display = false;
              }
            }
          }
        }
      }
    }
  }


  // Detect if the chart button has been clicked
  void checkChartButton() {
    if (mouseX >= b_chart.m_screenX - 10 && mouseX <= b_chart.m_screenX + 10 && 
      mouseY >= b_chart.m_screenY - 10 && mouseY <= b_chart.m_screenY + 10 ) {
      makeChoice = false;
      chart.calculateTotals();
    }
  }
}

